import uuid

from datetime import datetime, timezone
from typing import Optional, List

from sqlalchemy import Column, JSON
from sqlmodel import Field

from ..general.models import VacanciesOrganizationModel


class Feed(VacanciesOrganizationModel, table=True):
    name: str = Field(index=True)
    status: str = Field(index=True)
    file_url: Optional[str] = Field()
    ats_link_id: Optional[uuid.UUID] = Field()
    last_sync_date: Optional[datetime] = Field()
    synced_vacancy_count: int = Field(default=0)
    mapping: dict = Field(sa_column=Column(JSON))
    custom_fields: dict = Field(sa_column=Column(JSON))


class Vacancy(VacanciesOrganizationModel, table=True):
    feed_id: uuid.UUID = Field(foreign_key="feed.id")
    reference_number: str = Field(index=True)
    requisition_id: str = Field(index=True)
    title: str = Field(index=True)
    description: str = Field()
    status: str = Field()
    job_site_url: str = Field()
    company_name: str = Field(index=True)
    parent_company_name: Optional[str] = Field()
    remote_type: Optional[str] = Field()
    publish_date: Optional[datetime] = Field()
    expiration_date: Optional[datetime] = Field()
    last_updated_date: Optional[datetime] = Field()
    category: List[str] = Field(sa_column=Column(JSON))
    experience: List[str] = Field(sa_column=Column(JSON))
    education: List[str] = Field(sa_column=Column(JSON))
    hours_fte: Optional[float] = Field(default=1.0)
    hours_min: Optional[int] = Field(default=1)
    hours_max: Optional[int] = Field(default=40)
    location_address: Optional[str] = Field()
    location_zipcode: Optional[str] = Field()
    location_city: Optional[str] = Field()
    location_state: Optional[str] = Field()
    location_country: Optional[str] = Field()
    salary_min: Optional[float] = Field()
    salary_max: Optional[float] = Field()
    salary_currency: str = Field(default="EUR")
    salary_frequency: str = Field(default="month")
    recruiter_first_name: Optional[str] = Field()
    recruiter_last_name: Optional[str] = Field()
    recruiter_phone_number: Optional[str] = Field()
    recruiter_email: Optional[str] = Field()
    recruiter_role: Optional[str] = Field()


class Modification(VacanciesOrganizationModel, table=True):
    name: str = Field(index=True)
    type: str = Field(index=True)
    selector_configuration: dict = Field(sa_column=Column(JSON))
    changes_configuration: dict = Field(sa_column=Column(JSON))




from dataclasses import field
from datetime import datetime

from pydantic.dataclasses import dataclass


@dataclass
class TalentroCandidate:
    id: str
    email: str = ""
    first_name: str = ""
    last_name: str = ""
    phone_number: str = ""
    hashed_email: str = ""
    cv: str = ""
    motivation_letter: str = ""
    linked_in: str = ""
    created_at: datetime = field(default_factory=datetime.now)
    updated_at: datetime = None


from dataclasses import field
from datetime import datetime

from pydantic.dataclasses import dataclass

from talentro.dataclasses.candidate import TalentroCandidate
from talentro.dataclasses.vacancy import TalentroVacancy


@dataclass
class TalentroApplication:
    id: str
    status: str
    source: str
    candidate: TalentroCandidate = field(default_factory=TalentroCandidate)
    vacancy: TalentroVacancy = field(default_factory=TalentroVacancy)
    created_at: datetime = field(default_factory=datetime.now)
    updated_at: datetime = None


from dataclasses import field
from datetime import datetime
from typing import Optional, List

from pydantic.dataclasses import dataclass

from talentro.vacancies.models import Vacancy, Feed


@dataclass
class TalentroVacancyLocation:
    #  Required fields
    zip_code: str
    city: str

    # Optional fields
    address: str | None = None
    state: str | None = None
    country: str | None = None


@dataclass
class TalentroSalary:
    min: float | None = None
    max: float | None = None
    currency: str = "EUR"
    frequency: str = "month"


@dataclass
class TalentroHours:
    min: int = 0
    max: int = 40
    fte: float = 1.0


@dataclass
class TalentroContactDetails:

    # Required fields
    email: str | None = None
    first_name: str | None = None
    last_name: str | None = None
    phone_number: str | None = None
    role: str | None = None



@dataclass
class TalentroVacancy:

    # Required fields
    reference_number: str
    requisition_id: str
    title: str
    description: str
    job_site_url: str
    company_name: str
    publish_date: datetime | None = None
    category: List[str] = field(default_factory=list)
    experience: List[str] = field(default_factory=list)
    education: List[str] = field(default_factory=list)

    # Connected data
    hours: TalentroHours = field(default_factory=TalentroHours)
    location: TalentroVacancyLocation = field(default_factory=TalentroVacancyLocation)
    salary: TalentroSalary = field(default_factory=TalentroSalary)
    recruiter: TalentroContactDetails = field(default_factory=TalentroContactDetails)

    # Optional fields
    status: str | None = None
    parent_company_name: str | None = None
    remote_type: str | None = None
    expiration_date: datetime | None = None
    last_updated_date: datetime | None = None

    def convert_to_model(self, feed: Feed):
        return Vacancy(
            feed_id=feed.id,
            reference_number=self.reference_number,
            requisition_id=self.requisition_id,
            title=self.title,
            description=self.description,
            status=self.status,
            job_site_url=self.job_site_url,
            company_name=self.company_name,
            parent_company_name=self.parent_company_name,
            remote_type=self.remote_type,
            publish_date=self.publish_date,
            expiration_date=self.expiration_date,
            last_updated_date=self.last_updated_date,
            category=self.category,
            experience=self.experience,
            education=self.education,
            hours_fte=self.hours.fte,
            hours_min=self.hours.min,
            hours_max=self.hours.max,
            location_address=self.location.address,
            location_zipcode=self.location.zip_code,
            location_city=self.location.city,
            location_state=self.location.state,
            location_country=self.location.country,
            salary_min=self.salary.min,
            salary_max=self.salary.max,
            salary_currency=self.salary.currency,
            salary_interval=self.salary.interval,
            recruiter_first_name=self.recruiter.first_name,
            recruiter_last_name=self.recruiter.last_name,
            recruiter_phone_number=self.recruiter.phone_number,
            recruiter_email=self.recruiter.email,
            recruiter_role=self.recruiter.role,
        )


import uuid

from datetime import datetime, timezone
from typing import Optional
from sqlalchemy import PrimaryKeyConstraint, Identity, Integer

from sqlalchemy import Column
from sqlmodel import SQLModel, Field

from ..general.models import BillingOrganizationModel


class BillingEvent(SQLModel, table=True):
    id: Optional[int] = Field(
        default=None,
        sa_column=Column(Integer, Identity(), nullable=False)
    )

    organization: uuid.UUID = Field(index=True)
    event_time: Optional[datetime] = Field(
        default_factory=lambda: datetime.now(timezone.utc),
        nullable=False,
    )
    sku: str = Field(nullable=False)

    __table_args__ = (
        PrimaryKeyConstraint("id", "event_time"),
    )


class BillingProfile(BillingOrganizationModel, table=True):
    stripe_customer_id: str = Field(nullable=False)
